LDEVEL=/usr/lib/linuxconf-devel/
USR_BIN=$(RPM_BUILD_ROOT)/usr/bin
USR_INCLUDE=$(RPM_BUILD_ROOT)/usr/include/tlmp

TLMP_LIB=$(RPM_BUILD_ROOT)/usr/lib/tlmp
USR_LIB=$(RPM_BUILD_ROOT)/usr/lib/
FLAGS=-funsigned-char -g -O2 -Wall 
EGCS=$(shell test -f /usr/bin/egcs && /bin/echo -n 1)
ifeq ($(EGCS),1)
# RedHat has decided that egcs was for rh6.2 compatibility
# instead of using an environment variable. Very odd
# On rh7, we will use gcc. At some point egcs will be history anyway
RH7=$(shell grep "release 7" /etc/redhat-release 2>/dev/null | wc -l)
RH7_=$(shell expr $(RH7))
ifeq ($(RH7_),1)
GPPLINK=gcc
GPP=gcc
else
GPPLINK=egcs
GPP=egcs
endif
else
GPP=g++
# Using g++ to link pulls libg++ even if the program does not need it
GPPLINK=gcc
endif

CFLAGS= -DPUBLIC= -DPRIVATE= -DPROTECTED=

CPP_DEFINES=-Dexport= -DSTATIC= -DEXPORT= -DPUBLIC= -DPROTECTED= -DPRIVATE= -DVIRTUAL= \
	-Dnear= -Dfar=  -D_GNU_SOURCE -DUNIX -DTOOLS_VERSION=$(TOOLS_VERSION)

.cc.o:
	@echo "    " $<
	@$(GPP) $(CFLAGS) $(FLAGS) $(CPP_DEFINES) \
		-I/usr/include/tlmp \
		-I/usr/include/linuxconf -c $< -o $@

.cc.os:
	@echo "    " $<
	@$(GPP) -fPIC $(CFLAGS) $(FLAGS) $(CPP_DEFINES) \
		-I/usr/include/tlmp \
		-I/usr/include/linuxconf -c $< -o $@


.SUFFIXES: .tlcc .cc .os
.tlcc.cc:
	tlcc $< $@

lib: $(CURDIR).a $(CURDIR).so

$(CURDIR).a: $(OBJS)
	ar cr $(CURDIR).a $(OBJS)
	ranlib $(CURDIR).a

$(CURDIR).so: $(OBJSH)
	$(GPPLINK) -g -shared -Wl,-soname,$(CURDIR).so.1 -o \
		$(CURDIR).so \
		$(OBJSH) $(EXTRAMODLIBS)

msg:
	$(LDEVEL)/msgscan $(CURDIR) \
		$(CURDIR).dic $(CURDIR).m EF *.cc *.tlcc

msg.clean:
	$(LDEVEL)/msgclean $(CURDIR).dic
	$(LDEVEL)/msgscan $(CURDIR) \
		$(CURDIR).dic $(CURDIR).m EF *.cc *.tlcc

msg.eng:
	mkdir -p $(TLMP_LIB)/help.eng
	$(LDEVEL)/msgcomp -p./ \
		$(TLMP_LIB)/help.eng/$(CURDIR).eng eE \
		$(CURDIR)

msg.fr:
	mkdir -p $(TLMP_LIB)/help.fr
	$(LDEVEL)/msgcomp -p./ \
		$(TLMP_LIB)/help.fr/$(CURDIR).fr TFeE \
		$(CURDIR)

ifdef DIRS
clean: $(LOCAL_CLEAN)
	rm -f x *.o *.obt *.bak .bak *~ *.a *.old *.log *.nap *.nar core
	@for i in $(DIRS); do make -C $$i clean  || exit 1; done
else
clean: $(LOCAL_CLEAN)
	rm -f x *.o *.obt *.bak *~ *.a *.old *.log *.nap *.nar
endif

all:
ifdef DIRS
	@for i in $(DIRS); do make GCC=$(GCC) GPP=$(GPP) -C $$i $@ || exit 1; done
endif


ifdef DIRS
install: $(LOCAL_INSTALL) install_subdirs $(LOCAL_POSTINSTALL)

install_subdirs:
	@echo doing subdirs
	@for i in $(DIRS); do make -C $$i install  || exit 1 ; done
	@echo end subdirs

else
install: $(LOCAL_INSTALL) $(LOCAL_POSTINSTALL)
endif


pubspec_xml:
	@mkdir -p /tmp/xml
	nacom -x *cc >/tmp/xml/$(CURDIR).xml

